#include	"quality.cpp"
#include	<vector>

class	bamqc{
private:
	vector<char>	ref;
	string	dir;
	Quality	qual;
	vector<string>	bam_file;
	
	void	add_list(cchr	*F);
	void	load_ref(cchr	*F);
	void	one_bam(cchr	*F);
public:
	void	document(void);
	void	options(int	ac,	char	**av);
};

void	bamqc::add_list(cchr	*F){
	FILE	*f=fopen(F,	"rt");
	if(f==NULL){	fprintf(stderr,	"can't open %s\n",	F);	return;	}
	char	buff[1024],	temp[1024];
	while(fgets(buff,	1024,	f)!=NULL){
		sscanf(buff,	"%s",	temp);	bam_file.push_back(temp);
	}
	fclose(f);
}

void	bamqc::load_ref(cchr	*F){
	ref.clear();
	FILE	*f=fopen(F,	"rt");
	if(f==NULL){	fprintf(stderr,	"can't open %s\n",	F);	exit(0);	}
	char	buff[1024];
	while(fgets(buff,	1024,	f)!=NULL)	if(buff[0]!='>'){
		for(char	*p=buff;	*p>=33;	p++)	ref.push_back(*p);
	}	
	fclose(f);
}

void	bamqc::one_bam(cchr	*F){
	printf("%s\n",	F);	fflush(stdout);
	samfile_t	*file=samopen(F, "rb", 0);
	if(file==NULL){	fprintf(stderr,	"can't open %s\n",	F);	return;	}
	bam1_t	*b=bam_init1();
	qual.group.clear();
	
	int32_t	tid=-1;
	while(samread(file, b)>= 0){
		if(b->core.tid<0||b->core.tid>=file->header->n_targets)	continue;
		if(b->core.tid!=tid){
			tid=b->core.tid;
			string	s=file->header->target_name[b->core.tid];
			s=dir+s+".fa";	load_ref(s.c_str());
			fprintf(stderr,	"%s\n",	s.c_str());
			qual.refs=&ref[0];	qual.refl=ref.size();
		}
		qual.push(b);
	}
	bam_destroy1(b);
	samclose(file);
	qual.save(F);
}

void	bamqc::document(void){
	fprintf(stderr,	"\nname	bamqc 0.1");
	fprintf(stderr,	"\nfunc	show summary information of each bam");
	fprintf(stderr,	"\nauthor	Yi Wang @ Fuli Yu' Group @ BCM");
	fprintf(stderr,	"\nusage	bamqc [option] <ref_dir> [bam1 bam2...]");
	fprintf(stderr,	"\n	-l <file>	batch BAMs");	
	fprintf(stderr,	"\n	-m <mask>	filter read by mask (1796)");
	fprintf(stderr,	"\n	-o <oper>	filter read by  operations rate > orate (0.15)");
	fprintf(stderr,	"\n\n");
	exit(0);
}

void	bamqc::options(int	ac,	char	**av){
	qual.mask=1796;	qual.oper=0.15;
	char	opt;
	while((opt=getopt(ac,	av,	"l:m:o:"))>=0){
		switch(opt) {
		case	'l':	add_list(optarg);	break;
		case	'm':	qual.mask=atoi(optarg);	break;
		case	'o':	qual.oper=atof(optarg);	break;
		default:	document();
		}
	}
	if(optind+1>ac)	document();
	dir=av[optind];	if(dir[dir.size()-1]!='/')	dir+='/';
	for(int	i=optind+1;	i<ac;	i++)	bam_file.push_back(av[i]);
	for(uint	i=0;	i<bam_file.size();	i++)	one_bam(bam_file[i].c_str());
}

int	main(int	ac,	char	**av){
	bamqc	b;
	b.options(ac,	av);
	return	0;
}
