#include	<algorithm>
#include	"model.cpp"
#include	<iostream>
#include	<unistd.h>
#include	<cstdlib>
#include	<cstring>
#include	<fstream>
#include	<sstream>
#include	<ctype.h>
#include	<cfloat>
#include	<zlib.h>
using	namespace	std;
const	uint	bufn=64*1024;
const	char	acgt[5]="ACGT";

class	SNProb{
private:
	fast	cut,	bias;
	uint	bin;
	string	gender_file,	in_file;
	vector<string>	bam_name,	name;
	vector<uint>	bam2id;
	vector<Model>	site;
	uint32_t	bn,	in,	mn;
	gzFile	fbin;

	string	get_name(string	&S);
	void	load_head(gzFile	F);
	void	fill_gender(void);
	void	estimate(uint	N);
public:
	void	document(void);
	void	options(int	ac,	char	**av);
	void	work(void);	
};

void	SNProb::document(void){
	fprintf(stderr,	"\nname	snprob 0.1");
	fprintf(stderr,	"\nfunc	population based SNP detection");
	fprintf(stderr,	"\nauthor	Yi Wang @ Fuli Yu' Group");
	fprintf(stderr,	"\nusage	SNProb [options] <gender.txt in.gz>");
	fprintf(stderr,	"\n	-n <INT>	bin size for genotype calling (20480)");
	fprintf(stderr,	"\n	-s <FLT>	set significance level (2)");
	fprintf(stderr,	"\n	-m <FLT>	set mapping bias cutoff (1e-6)");
	fprintf(stderr,	"\n");
	fprintf(stderr,	"\n	-a <FLT>	set AA prior alt allele rate (0.995)");
	fprintf(stderr,	"\n	-h <FLT>	set AB prior alt allele rate (0.500)");
	fprintf(stderr,	"\n	-b <FLT>	set BB prior alt allele rate (0.010)");	
	fprintf(stderr,	"\n	-p <FLT>	set prior alt rate confidence (100)");
	fprintf(stderr,	"\n	-f <FLT>	set Fst of the sample (0.13)");
	fprintf(stderr,	"\n	-o <FLT>	set outlier probability (1e-6)");
	fprintf(stderr,	"\n\n");
	exit(0);
}

void	SNProb::options(int	ac,	char	**av){
	bin=20480;	cut=2;	bias=1e-6;
	char	opt;
	while((opt=getopt(ac,	av,	"n:s:m:a:h:b:p:f:o:"))>=0){
		switch(opt) {
		case	'n':	bin=atoi(optarg);	break;
		case	's':	cut=atof(optarg);	break;
		case	'm':	bias=atof(optarg);	break;
		
		case	'a':	Model::maa=atof(optarg);	break;
		case	'h':	Model::mab=atof(optarg);	break;		
		case	'b':	Model::mbb=atof(optarg);	break;	
		case	'p':	Model::pre=atof(optarg);	break;
		case	'f':	Model::fst=atof(optarg);	break;
		case	'o':	Model::out=atof(optarg);	break;
		default:	document();
		}
	}
	if(ac<optind+2)	document();
	gender_file=av[optind];
	in_file=av[optind+1];
}

void	SNProb::load_head(gzFile	F){
	gzread(F,	&bn,	4);
	for(uint	i=0;	i<bn;	i++){
		uint32_t	n;
		char	temp[1024];
		gzread(F,	&n,	4);
		gzread(F,	temp,	n);	temp[n]=0;
		bam_name.push_back(temp);
		uint	p=0;
		for(uint	j=strlen(temp);	j;	j--)	if(temp[j-1]=='/'){	p=j;	break;	}
		string	s;
		for(uint	j=p;	temp[j]!='.'&&temp[j];	j++)	s.push_back(temp[j]);
		name.push_back(s);
	}
	sort(name.begin(),	name.end());
	in=unique(name.begin(),	name.end())-name.begin();
	bam2id.resize(bn);
	for(uint	i=0;	i<bn;	i++){
		for(uint	j=0;	j<in;	j++)	if(bam_name[i].find(name[j])!=string::npos){
			bam2id[i]=j;	break;
		}
	}	
}

void	SNProb::fill_gender(void){
	Model::gen.assign(in,	2);
	ifstream	fi(gender_file.c_str());
	if(!fi){	cerr<<"can't open "<<gender_file<<endl;	exit(0);	}
	char	buffer[1024];
	for(fi.getline(buffer,	1024);	!fi.eof();	fi.getline(buffer,	1024)){
		istringstream	si(buffer);
		string	temp;	si>>temp;
		for(uint	i=0;	i<in;	i++)	if(name[i]==temp){
			if(strstr(buffer,	"female")!=NULL)	Model::gen[i]=Gender_Female;
			else	if(strstr(buffer,	"male")!=NULL)	Model::gen[i]=Gender_Male;
		}
	}
	fi.close();
	for(uint	i=0;	i<in;	i++)	if(Model::gen[i]==2){
		Model::gen[i]=Gender_Male;
		cerr<<"unknown gender:\t"<<name[i]<<endl;
	}
}

void	SNProb::estimate(uint	N){
	#pragma omp parallel for
	for(uint	i=0;	i<N;	i++)	site[i].estimate();

	for(uint	i=0;	i<N;	i++)	if(site[i].sig>=cut){
		cout<<site[i].pos<<'\t'<<site[i].all<<endl;
		if(!(mn%bin)){
			gzclose(fbin);	fbin=Z_NULL;
			char	temp[256];	sprintf(temp,	"%s_%09u.bin",	site[i].chr,	site[i].pos);
			fbin=gzopen(temp,	"wt");
			gzprintf(fbin,	"chr\tpos\tallele");
			for(uint	j=0;	j<in;	j++)	gzprintf(fbin,	"\t%s",	name[j].c_str());
		}
		site[i].write(fbin);	
		mn++;
	}
	fprintf(stderr,	"%.1fM\r",	1e-6*site[N-1].pos);
}

void	SNProb::work(void){
	gzFile	f=gzopen(in_file.c_str(),	"rb");
	if(f==Z_NULL)	exit(0);
	load_head(f);
	fill_gender();
	site.resize(bufn);
	fbin=Z_NULL;	mn=0;
	
	uint	i;
	vector<uint16_t>	buff;	buff.resize(2*bn);
	for(i=0;	!gzeof(f);){
		gzread(f,	site[i].chr,	16);
		gzread(f,	&site[i].pos,	4);			
		uint8_t	temp;
		gzread(f,	&temp,	1);	site[i].all[0]=acgt[temp];
		gzread(f,	&temp,	1);	site[i].all[1]=acgt[temp];
		site[i].all[2]=0;
		fast	pstr,	ppos;
		gzread(f,	&pstr,	4);	gzread(f,	&ppos,	4);
		gzread(f,	&buff[0],	4*bn);
		if(pstr*ppos<bias)	continue;
		site[i].cnt.assign(2*in,	0);
		for(uint	j=0;	j<bn;	j++){
			site[i].cnt[bam2id[j]*2]+=0.01*buff[j*2];
			site[i].cnt[bam2id[j]*2+1]+=0.01*buff[j*2+1];
		}
		if(i+1==bufn)	estimate(bufn);
		i=(i+1)%bufn;
	}
	estimate(i);
	gzclose(f);	gzclose(fbin);
}

int	main(int	ac,	char	**av){
	SNProb	d;
	d.options(ac,	av);
	d.work();
	return	0;
}

